@@ -10,6 +10,7 @@ import os
 import select
 import socket
 import ssl
+import sys
 
 import gunicorn.http as http
 import gunicorn.http.wsgi as wsgi
@@ -17,16 +18,43 @@ import gunicorn.util as util
 import gunicorn.workers.base as base
 from gunicorn import six
 
+class StopWaiting(Exception):
+    """ exception raised to stop waiting for a connnection """
 
 class SyncWorker(base.Worker):
 
-    def run(self):
-        # self.socket appears to lose its blocking status after
-        # we fork in the arbiter. Reset it here.
-        for s in self.sockets:
-            s.setblocking(0)
+    def accept(self, listener):
+        client, addr = listener.accept()
+        client.setblocking(1)
+        util.close_on_exec(client)
+        self.handle(listener, client, addr)
 
-        ready = self.sockets
+    def wait(self, timeout):
+        try:
+            self.notify()
+            ret = select.select(self.wait_fds, [], [], timeout)
+            if ret[0]:
+                return ret[0]
+
+        except select.error as e:
+            if e.args[0] == errno.EINTR:
+                return self.sockets
+            if e.args[0] == errno.EBADF:
+                if self.nr < 0:
+                    return self.sockets
+                else:
+                    raise StopWaiting
+            raise
+
+    def is_parent_alive(self):
+        # If our parent changed then we shut down.
+        if self.ppid != os.getppid():
+            self.log.info("Parent changed, shutting down: %s", self)
+            return False
+        return True
+
+    def run_for_one(self, timeout):
+        listener = self.sockets[0]
         while self.alive:
             self.notify()
 
@@ -34,54 +62,71 @@ class SyncWorker(base.Worker):
             # that no connection is waiting we fall down to the
             # select which is where we'll wait for a bit for new
             # workers to come give us some love.
+            try:
+                self.accept(listener)
+                # Keep processing clients until no one is waiting. This
+                # prevents the need to select() for every client that we
+                # process.
+                continue
 
-            for sock in ready:
-                try:
-                    client, addr = sock.accept()
-                    client.setblocking(1)
-                    util.close_on_exec(client)
-                    self.handle(sock, client, addr)
+            except EnvironmentError as e:
+                if e.errno not in (errno.EAGAIN, errno.ECONNABORTED,
+                        errno.EWOULDBLOCK):
+                    raise
 
-                    # Keep processing clients until no one is waiting. This
-                    # prevents the need to select() for every client that we
-                    # process.
-                    continue
-
-                except socket.error as e:
-                    if e.args[0] not in (errno.EAGAIN, errno.ECONNABORTED,
-                            errno.EWOULDBLOCK):
-                        raise
-
-            # If our parent changed then we shut down.
-            if self.ppid != os.getppid():
-                self.log.info("Parent changed, shutting down: %s", self)
+            if not self.is_parent_alive():
                 return
 
             try:
-                self.notify()
-                ret = select.select(self.sockets, [], self.PIPE, self.timeout)
-                if ret[0]:
-                    ready = ret[0]
-                    continue
-            except select.error as e:
-                if e.args[0] == errno.EINTR:
-                    ready = self.sockets
-                    continue
-                if e.args[0] == errno.EBADF:
-                    if self.nr < 0:
-                        ready = self.sockets
+                self.wait(timeout)
+            except StopWaiting:
+                return
+
+    def run_for_multiple(self, timeout):
+        while self.alive:
+            self.notify()
+
+            try:
+                ready = self.wait(timeout)
+            except StopWaiting:
+                return
+
+            if ready is not None:
+                for listener in ready:
+                    if listener == self.PIPE[0]:
                         continue
-                    else:
-                        return
-                raise
+
+                    try:
+                        self.accept(listener)
+                    except EnvironmentError as e:
+                        if e.errno not in (errno.EAGAIN, errno.ECONNABORTED,
+                                errno.EWOULDBLOCK):
+                            raise
+
+            if not self.is_parent_alive():
+                return
+
+    def run(self):
+        # if no timeout is given the worker will never wait and will
+        # use the CPU for nothing. This minimal timeout prevent it.
+        timeout = self.timeout or 0.5
+
+        # self.socket appears to lose its blocking status after
+        # we fork in the arbiter. Reset it here.
+        for s in self.sockets:
+            s.setblocking(0)
+
+        if len(self.sockets) > 1:
+            self.run_for_multiple(timeout)
+        else:
+            self.run_for_one(timeout)
 
     def handle(self, listener, client, addr):
         req = None
         try:
             if self.cfg.is_ssl:
                 client = ssl.wrap_socket(client, server_side=True,
-                        do_handshake_on_connect=False,
-                        **self.cfg.ssl_options)
+                    **self.cfg.ssl_options)
 
             parser = http.RequestParser(self.cfg, client)
             req = six.next(parser)
@@ -97,11 +142,14 @@ class SyncWorker(base.Worker):
             else:
                 self.log.debug("Error processing SSL request.")
                 self.handle_error(req, client, addr, e)
-        except socket.error as e:
-            if e.args[0] != errno.EPIPE:
-                self.log.exception("Error processing request.")
+        except EnvironmentError as e:
+            if e.errno not in (errno.EPIPE, errno.ECONNRESET):
+                self.log.exception("Socket error processing request.")
             else:
-                self.log.debug("Ignoring EPIPE")
+                if e.errno == errno.ECONNRESET:
+                    self.log.debug("Ignoring connection reset")
+                else:
+                    self.log.debug("Ignoring EPIPE")
         except Exception as e:
             self.handle_error(req, client, addr, e)
         finally:
@@ -136,22 +184,21 @@ class SyncWorker(base.Worker):
             finally:
                 if hasattr(respiter, "close"):
                     respiter.close()
-        except socket.error:
-            raise
-        except Exception as e:
-            if resp.headers_sent:
+        except EnvironmentError:
+            # pass to next try-except level
+            six.reraise(*sys.exc_info())
+        except Exception:
+            if resp and resp.headers_sent:
                 # If the requests have already been sent, we should close the
                 # connection to indicate the error.
+                self.log.exception("Error handling request")
                 try:
                     client.shutdown(socket.SHUT_RDWR)
                     client.close()
-                except socket.error:
+                except EnvironmentError:
                     pass
-
-                return
-            # Only send back traceback in HTTP in debug mode.
-            self.handle_error(req, client, addr, e)
-            return
+                raise StopIteration()
+            raise
         finally:
             try:
                 self.cfg.post_request(self, req, environ, resp)
